.\" Automatically generated by Pandoc 2.9.2.1
.\"
.TH "parse_args" "3" "0.3.4" "0.3.4" "Advanced argument parsing for Tcl"
.hy
.SH NAME
.PP
parse_args - Core-style argument parsing for scripts
.SH SYNOPSIS
.PP
\f[B]package require parse_args\f[R] ?0.3.4?
.PP
\f[B]parse_args::parse_args\f[R] \f[I]args\f[R] \f[I]argspec\f[R]
?\f[I]varname\f[R]?
.SH DESCRIPTION
.PP
The commands provided by the Tcl core follow a pattern of optional,
named arguments (named with a leading \[lq]-\[rq] to indicate an option)
in addition to positional arguments.
They strongly prefer named arguments when the number of arguments and
particularly optional arguments grows beyond about 3.
Tcl scripts however have no access to standard argument parsing
facilities to implement this pattern and have to build their named
argument parsing machinery from scratch.
This is slow to do in script, error prone, and creates noise in the code
that distracts from the core task of the command implementation.
This means that script-defined commands generally present half-baked
argument handling APIs and tend to over-use positional arguments.
.PP
This package aims to address this weakness by providing a robust
argument parsing mechanism to script code that can implement the full
set of patterns established by the core Tcl commands, is terse and
readable to specify the arguments, and is fast enough to use in any
situation.
.SH COMMANDS
.TP
\f[B]parse_args::parse_args\f[R] \f[I]args\f[R] \f[I]argspec\f[R]
Parse the arguments supplied in \f[I]args\f[R] against the specification
given in \f[I]argspec\f[R], setting variables in the current scope as
defined in the \f[I]argspec\f[R].
If the optional \f[I]varname\f[R] argument is supplied, it names a
variable that will have a dictionary written to it containing the parsed
arguments and no variables will be created for the arguments.
See \f[B]ARGUMENT SPECIFICATION\f[R] below for the format of
\f[I]argspec\f[R].
.SH ARGUMENT SPECIFICATION
.PP
The syntax of the argument specification (given in the \f[I]argspec\f[R]
argument to the \f[B]parse_args\f[R] command) is a dictionary, with the
keys corresponding to the names by which the arguments will be available
to the caller, either as variables created by the \f[B]parse_args\f[R]
command or keys in the dictionary written to \f[I]varname\f[R] if that
was specified.
If the first character of the key is a \[lq]-\[rq] character it defines
a named option, otherwise a positional parameter.
Like in proc argument definitions the name \f[C]args\f[R] is special,
and will consume all remaining arguments.
.PP
The values of the dictionary contain the definition of the corresponding
argument, such as \f[C]-required\f[R], which marks that argument as
being required (whether a named or positional argument).
The options that are available to define the argument are:
.TP
\f[B]-default\f[R] \f[I]val\f[R]
Provides a default value for the argument if it wasn\[cq]t supplied.
Without this an argument that wasn\[cq]t supplied won\[cq]t have its
corresponding variable (or dictionary key) set.
\f[I]val\f[R] is permitted to be a value that is outside of the range of
possible values that would be accepted from the caller, by design.
.TP
\f[B]-required\f[R]
If this boolean option is given, the argument is flagged as required.
Failing to supply the arg causes the \f[B]parse_args\f[R] to throw an
exception similar to what would happen if a required positional argument
to a proc was not provided.
.TP
\f[B]-validate\f[R] \f[I]cmdprefix\f[R]
Append the value supplied for this argument to the \f[I]cmdprefix\f[R]
and run it.
The value is rejected if an exception is thrown or the value returned
isn\[cq]t a Tcl boolean true value.
.TP
\f[B]-name\f[R] \f[I]newname\f[R]
Change the name of the variable / dictionary key that will receive the
value for this argument to \f[I]newname\f[R].
By default the name will be the argspec key (without the leading
\[lq]-\[rq] for named arguments).
.TP
\f[B]-boolean\f[R]
Flag this argument as a boolean, similar to \f[B]-nocase\f[R] to
\f[C]regexp\f[R].
The variable will always be defined and be a boolean, true if the
argument was supplied and false if it wasn\[cq]t.
.TP
\f[B]-args\f[R] \f[I]count\f[R]
By default named arguments consume a single following argument as the
value for that argument.
This setting changes that to \f[I]count\f[R], which will cause the
following \f[I]count\f[R] arguments to be gathered into the variable
holding this argument.
.TP
\f[B]-enum\f[R] \f[I]possible_values\f[R]
Restrict the values that will be accepted for this argument to those in
the \f[I]possible_values\f[R] list.
Similar to using a validator that checks for inclusion in that list but
with additional internal performance optimizations.
.TP
\f[B]-#\f[R] \f[I]free_form_text\f[R]
Embed a comment for this argument.
The supplied \f[I]free_form_text\f[R] is ignored by \f[B]parse_args\f[R]
and serves only to document the argument to programmers reading the
code.
In some future version this text may be incorporated into a usage error
message that is generated when argument parsing fails.
.TP
\f[B]-multi\f[R] \f[I]name\f[R]
Group a set of defined arguments together, so they act like mutually
exclusive flags.
Used to implement patterns like the arguments \f[C]-ascii\f[R],
\f[C]-dictionary\f[R], \f[C]-integer\f[R] and \f[C]-real\f[R] arguments
to the core \f[B]lsort\f[R] command.
\f[I]name\f[R] will be the name of the variable that holds the value of
the flag that was supplied.
A \f[B]-default\f[R] setting on any of the linked \f[B]-multi\f[R]
arguments (those that share a \f[I]name\f[R]) will supply a default
value in \f[I]name\f[R] if none were passed.
A \f[B]-required\f[R] setting on any of the linked \f[B]-multi\f[R]
arguments will require that the caller supply at least one of them.
If multiple different flags were given, the one last in the argument
list applies.
.TP
\f[B]-alias\f[R]
Treat the value supplied for this argument as a variable name and bind
the resulting variable for this argument to the variable of that name in
the call frame above this one (upvar 1).
.SH EXAMPLES
.PP
Mimic the argument handling of the core \f[C]glob\f[R] command:
.IP
.nf
\f[C]
proc glob args {
    parse_args::parse_args $args {
        -directory  {}
        -join       {-boolean}
        -nocomplain {-boolean}
        -path       {}
        -tails      {-boolean}
        -types      {-default {}}
        args        {-name patterns}
    }

    if {$join} {
        set patterns    [list [file join {*}$patterns]]
    }

    if {[llength $patterns] == 0 && $nocomplain} return

    foreach pattern $patterns {
        if {[info exists directory]} {
            ...
        }
    }
    ...
}
\f[R]
.fi
.PP
Mimic \f[C]regex\f[R]:
.IP
.nf
\f[C]
proc regexp args {
    parse_args::parse_args $args {
        -about      {-boolean}
        -expanded   {-boolean}
        -indices    {-boolean}
        -line       {-boolean}
        -linestop   {-boolean}
        -lineanchor {-boolean}
        -nocase     {-boolean}
        -all        {-boolean}
        -inline     {-boolean}
        -start      {-default 0}
        exp         {-required}
        string      {-required}
        matchvar    {}
        args        {-name submatchvars}
    }
    ...
}
\f[R]
.fi
.PP
\f[C]lsort\f[R]:
.IP
.nf
\f[C]
proc lsort args {
    parse_args::parse_args $args {
        -ascii      {-name compare_as -multi -default ascii}
        -dictionary {-name compare_as -multi}
        -integer    {-name compare_as -multi}
        -real       {-name compare_as -multi}

        -command    {}

        -increasing {-name order -multi -default increasing}
        -decreasing {-name order -multi}

        -indices    {-boolean}
        -index      {}
        -stride     {-default 1}
        -nocase     {-boolean}
        -unique     {-boolean}
        list        {-required}
    }

    if {![info exists command]} {
        switch -- $compare_as {
            ascii {
                set command {string compare}
                if {$nocase} {
                    lappend command -nocase
                }
            }
            dictionary {
                ...
            }
            integer - real {
                set command tcl::mathop::-
            }
        }
    }
}
\f[R]
.fi
.PP
\f[C]lsearch\f[R]:
.IP
.nf
\f[C]
proc lsearch args {
    parse_args::parse_args $args {
        -exact      {-name matchtype -multi}
        -glob       {-name matchtype -multi -default glob}
        -regexp     {-name matchtype -multi}

        -sorted     {-boolean}
        -all        {-boolean}
        -inline     {-boolean}
        -not        {-boolean}
        -start      {-default 0}

        -ascii      {-name compare_as -multi -default ascii}
        -dictionary {-name compare_as -multi}
        -integer    {-name compare_as -multi}
        -real       {-name compare_as -multi}

        -nocase     {-boolean}

        -decreasing {-name order -multi}
        -increasing {-name order -multi -default increasing}
        -bisect     {-boolean}

        -index      {}
        -subindices {-boolean}
    }

    if {$sorted && $matchtype in {glob regexp}} {
        error \[dq]-sorted is mutually exclusive with -glob and -regexp\[dq]
    }
}
\f[R]
.fi
.PP
A Tk widget - \f[C]entry\f[R]:
.IP
.nf
\f[C]
proc entry {widget args} {
    parse_args::parse_args $args {
        -disabledbackground {-default {}}
        -disabledforeground {-default {}}
        -invalidcommand     {-default {}}
        -readonlybackground {-default {}}
        -show               {}
        -state              {-default normal -enum {
            normal disabled readonly
        }}
        -validate           {-default none -enum {
            none focus focusin focusout key all
        }}
        -validatecommand    {-default {}}
        -width              {-default 0}
        -textvariable       {}
    }
}
\f[R]
.fi
.SH SEE ALSO
.PP
The paper presented at the 2016 Tcl Conference which discusses the
approach and design choices made in this package in much greater depth:
https://www.tcl-lang.org/community/tcl2016/assets/talk33/parse_args-paper.pdf
.SH BUGS
.PP
Please open an issue on the github tracker for the project if you
encounter any problems: https://github.com/RubyLane/parse_args/issues
.SH LICENSE
.PP
This package is Copyright 2022 Cyan Ogilvie, and is made available under
the same license terms as the Tcl Core
.SH AUTHORS
Cyan Ogilvie.
